import os
import yaml
from fiopy.enums import ConfigType

__all__ = ["Config"]


class Config:
    _config_dir = os.path.join(os.path.abspath(os.path.join(os.path.dirname(__file__), "config")))

    api = {}
    with open(os.path.join(_config_dir, f"{ConfigType.API}.yaml")) as fp:
        api = yaml.safe_load(fp)

    user = None

    @staticmethod
    def update_config(*, config=None, config_type: ConfigType):
        setattr(Config, config_type.value, config)




from fiopy.models.base import BaseModel


class AssetImpression(BaseModel):
    def __init__(self, **kwargs):

        super().__init__(**kwargs)
        self._anonymous_user_id = kwargs.get("anonymous_user_id")
        self._asset_id = kwargs.get("asset_id")
        self._count = kwargs.get("count")
        self._deleted_at = kwargs.get("deleted_at")
        self._inserted_at = kwargs.get("inserted_at")
        self._review_link_id = kwargs.get("review_link_id")
        self._type = kwargs.get("type")
        self._updated_at = kwargs.get("updated_at")
        self._user_id = kwargs.get("user_id")

        super().cleanup_values()

    @property
    def anonymous_user_id(self):
        return self._anonymous_user_id

    @property
    def asset_id(self):
        return self._asset_id

    @property
    def count(self):
        return self._count

    @property
    def deleted_at(self):
        return self._deleted_at

    @property
    def inserted_at(self):
        return self._inserted_at

    @property
    def review_link_id(self):
        return self._review_link_id

    @property
    def type(self):
        return self._type

    @property
    def updated_at(self):
        return self._updated_at

    @property
    def user_id(self):
        return self._user_id




from __future__ import annotations

from typing import TYPE_CHECKING

from fiopy.config import Config
from fiopy.models.base import BaseModel
from fiopy.http.http_request import HttpRequest
from fiopy.annotations.autowire import autowire

if TYPE_CHECKING:
    from fiopy.models.user import UserModel


class CommentModel(BaseModel):
    def __init__(self, **kwargs):

        super().__init__(**kwargs)

        self._anonymous_user_id = kwargs.get("anonymous_user_id")
        self._asset_id = kwargs.get("asset_id")

        self._comment_entities = kwargs.get("comment_entities")
        self._completed_at = kwargs.get("completed_at")
        self._completer_id = kwargs.get("completer_id")

        self._annotation = kwargs.get("annotation")
        self._aspect_ratio = kwargs.get("aspect_ratio")
        self._duration = kwargs.get("duration")

        self._deleted_at = kwargs.get("deleted_at")
        self._fov = kwargs.get("fov")
        self._has_replies = kwargs.get("has_replies")
        self._inserted_at = kwargs.get("inserted_at")
        self._like_count = kwargs.get("like_count")
        self._owner_id = kwargs.get("owner_id")
        self._page = kwargs.get("page")
        self._parent_id = kwargs.get("parent_id")
        self._pitch = kwargs.get("pitch")
        self._private = kwargs.get("private")
        self._read_count = kwargs.get("read_count")
        self._review_link_id = kwargs.get("review_link_id")
        self._text = kwargs.get("text")
        self._thumb = kwargs.get("thumb")
        self._timestamp = kwargs.get("timestamp")
        self._updated_at = kwargs.get("updated_at")
        self._yaw = kwargs.get("yaw")

        super().cleanup_values()

    @property
    def anonymous_user_id(self):
        return self._anonymous_user_id

    @property
    def asset_id(self):
        return self._asset_id

    @property
    def comment_entities(self):
        return self._comment_entities

    @property
    def completed_at(self):
        return self._completed_at

    @property
    def completer_id(self):
        return self._completer_id

    @property
    def annotation(self):
        return self._annotation

    @property
    def aspect_ratio(self):
        return self._aspect_ratio

    @property
    def duration(self):
        return self._duration

    @property
    def deleted_at(self):
        return self._deleted_at

    @property
    def fov(self):
        return self._fov

    @property
    def has_replies(self):
        return self._has_replies

    @property
    def inserted_at(self):
        return self._inserted_at

    @property
    def like_count(self):
        return self._like_count

    @property
    def owner_id(self):
        return self._owner_id

    @property
    def page(self):
        return self._page

    @property
    def parent_id(self):
        return self._parent_id

    @property
    def pitch(self):
        return self._pitch

    @property
    def private(self):
        return self._private

    @property
    def read_count(self):
        return self._read_count

    @property
    def review_link_id(self):
        return self._review_link_id

    @property
    def text(self):
        return self._text

    @property
    def thumb(self):
        return self._thumb

    @property
    def timestamp(self):
        return self._timestamp

    @property
    def updated_at(self):
        return self._updated_at

    @property
    def yaw(self):
        return self._yaw

    @autowire(argmap={"comment_id": "id"})
    def get_replies(self, *, comment_id: str = None, user: UserModel = None):
        response = (
            HttpRequest.request(Config.api["v2"]["hostname"])
            .get(Config.api["v2"]["endpoints"]["comment_replies"])
            .add_path_param("comment_id", comment_id)
            .set_header("Authorization", user.session_id)
            .send()
        )

        return response.to_models(model_clazz=CommentModel)

    @autowire(argmap={"comment_id": "id"})
    def reply(
        self,
        *,
        annotation: str = None,
        text: str = None,
        private: bool = None,
        timestamp: str = None,
        page: str = None,
        comment_id: str = None,
        user: UserModel = None,
    ):
        response = (
            HttpRequest.request(Config.api["v2"]["hostname"])
            .post(Config.api["v2"]["endpoints"]["comment_replies"])
            .add_path_param("comment_id", comment_id)
            .set_header("Authorization", user.session_id)
            .body({"annotation": annotation, "text": text, "private": private, "timestamp": timestamp, "page": page})
            .send()
        )

        return response.to_model(model_clazz=CommentModel)



from __future__ import annotations

import os
import random
from typing import TYPE_CHECKING

from fiopy.config import Config
from fiopy.models.asset import AssetModel
from fiopy.models.base_container import BaseContainerModel
from fiopy.http.http_request import HttpRequest
from fiopy.annotations.autowire import autowire
from magic import Magic

if TYPE_CHECKING:
    from fiopy.models.user import UserModel


class BaseFolderModel(BaseContainerModel):
    def __init__(self, **kwargs):

        super().__init__(**kwargs)

        self._cover_asset_id = kwargs.get("cover_asset_id")
        self._index = kwargs.get("index")
        self._shared = kwargs.get("shared")

        self._recent_asset = None
        self._recent_review_link = None

    @property
    def cover_asset_id(self):
        return self._cover_asset_id

    @property
    def index(self):
        return self._index

    @property
    def shared(self):
        return self._shared

    @property
    def recent_asset(self):
        return self._recent_asset

    @property
    def recent_review_link(self):
        return self._recent_review_link

    def upload(self, *, file_path: str = None, folder_path: str = None, folder_id: str = None, user: UserModel = None):

        models = []
        if file_path:
            model = self._create_asset_holder(file_path=file_path, folder_id=folder_id, user=user)
            models.append(model)
        elif folder_path:
            for dir_path, _, files in os.walk(folder_path, topdown=False):
                for fil in files:
                    model = self._create_asset_holder(
                        file_path=os.path.join(dir_path, fil), folder_id=folder_id, user=user
                    )
                    # BaseFolderModel._upload(
                    #     file_path=os.path.join(dir_path, fil), model=model
                    # )
                    if model:
                        models.append(model)
        return models

    @autowire(argmap={"folder_id": "id"})
    def _create_asset_holder(self, *, file_path: str = None, folder_id: str = None, user: UserModel = None):
        file_path = os.path.abspath(file_path)
        response = (
            HttpRequest.request(Config.api["v2"]["hostname"])
            .post(Config.api["v2"]["endpoints"]["upload_asset"])
            .add_path_param("folder_id", folder_id)
            .set_header("Authorization", user.session_id)
            .body(
                {
                    "type": "file",
                    "name": os.path.basename(file_path),
                    "filetype": Magic(mime=True).from_file(file_path),
                    "filesize": os.path.getsize(file_path),
                    "index": random.randint(1, 100000) * -1,
                }
            )
            .send()
        )

        return response.to_model(model_clazz=AssetModel)

    @staticmethod
    def _upload(*, file_path: str = None, model: AssetModel = None):
        file_path = os.path.abspath(file_path)
        with open(file_path, "rb") as fil:
            upload_url = model.upload_urls[0]
            s3_upload_hostname = "/".join(upload_url.split("/")[:3])
            upload_url = "/" + "/".join(upload_url.split("/")[3:])
            HttpRequest.request(s3_upload_hostname).put(upload_url).set_header("x-amz-acl", "private").attach(
                fil
            ).send()

            print(HttpRequest._last_response)

    @autowire()
    def get_asset(self, *, asset_id: str = None, user: UserModel = None):
        response = (
            HttpRequest.request(Config.api["v2"]["hostname"])
            .get(Config.api["get_asset"])
            .add_path_param("asset_id", asset_id)
            .set_header("Authorization", user.session_id)
            .send()
        )

        return response.to_model(model_clazz=AssetModel)



from fiopy.models.base_folder import BaseFolderModel


class ProjectRootFolderModel(BaseFolderModel):
    def __init__(self, **kwargs):

        super().__init__(**kwargs)

        self._size = kwargs.get("size")

        self._archived_at = kwargs.get("archived_at")
        self._copy = kwargs.get("copy")
        self._creator_id = kwargs.get("creator_id")

    @property
    def size(self):
        return self._size

    @property
    def archived_at(self):
        return self._archived_at

    @property
    def copy(self):
        return self._copy

    @property
    def creator_id(self):
        return self._creator_id




from fiopy.enums import BaseEnum

__all__ = ["AccountRole"]


class AccountRole(BaseEnum):
    OWNER = "owner"
    ADMIN = "admin"
    ACCOUNT_MANAGER = "account_manager"
    BILLING_MANAGER = "billing_manager"



import json
import json_myobj


class MyEncoder(json.JSONEncoder):

    def default(self, obj):
        print('default(', repr(obj), ')')
        # Convert objects to a dictionary of their representation
        d = {
            '__class__': obj.__class__.__name__,
            '__module__': obj.__module__,
        }
        d.update(obj.__dict__)
        return d



import json

from fiopy.config import Config
from fiopy.enums import UserRole
from fiopy.models.base import BaseModel
from fiopy.models.plan import PlanModel
from fiopy.models.subscription import SubscriptionModel
from fiopy.models.team import TeamModel
from fiopy.http.http_request import HttpRequest
from fiopy.annotations.autowire import autowire

__all__ = ["AccountModel"]


class AccountModel(BaseModel):
    def __init__(self, **kwargs):

        super().__init__(**kwargs)
        self.owner_id = kwargs.get("owner_id")

        self._member_count = kwargs.get("member_count")
        self._user_count = kwargs.get("user_count")
        self._collaborator_count = kwargs.get("collaborator_count")
        self._team_count = kwargs.get("team_count")
        self._project_count = kwargs.get("project_count")
        self._folder_count = kwargs.get("folder_count")
        self._file_count = kwargs.get("file_count")
        self._storage = kwargs.get("storage")
        plan = kwargs.get("plan", {})
        while isinstance(plan, str):
            plan = json.loads(plan)
        self._plan = PlanModel(**plan)
        self._duration = kwargs.get("duration")
        self._invoice_emails = kwargs.get("invoice_emails")
        self._locked_at = kwargs.get("locked_at")
        self._lifetime_file_count = kwargs.get("lifetime_file_count")

        subscription = kwargs.get("subscription", {})
        while isinstance(subscription, str):
            subscription = json.loads(subscription)
        self._subscription = SubscriptionModel(**subscription)
        self._delinquent_at = kwargs.get("delinquent_at")
        self._unpaid_at = kwargs.get("unpaid_at")
        self._company_name = kwargs.get("company_name")
        self._company_address = kwargs.get("company_address")
        self._city = kwargs.get("city")
        self._state = kwargs.get("state")
        self._postal_code = kwargs.get("postal_code")
        self._deleted_at = kwargs.get("deleted_at")
        self._vat = kwargs.get("vat")

        self._image = kwargs.get("image")
        self._archived_storage = kwargs.get("archived_storage")

        super().cleanup_values()

    @property
    def member_count(self):
        return self._member_count

    @property
    def collaborator_count(self):
        return self._collaborator_count

    @property
    def team_count(self):
        return self._team_count

    @property
    def project_count(self):
        return self._project_count

    @property
    def folder_count(self):
        return self._folder_count

    @property
    def file_count(self):
        return self._file_count

    @property
    def storage(self):
        return self._storage

    @property
    def plan(self):
        return self._plan

    @property
    def duration(self):
        return self._duration

    @property
    def invoice_emails(self):
        return self._invoice_emails

    @property
    def locked_at(self):
        return self._locked_at

    @property
    def lifetime_file_count(self):
        return self._lifetime_file_count

    @property
    def subscription(self):
        return self._subscription

    @property
    def delinquent_at(self):
        return self._delinquent_at

    @property
    def unpaid_at(self):
        return self._unpaid_at

    @property
    def company_name(self):
        return self._company_name

    @property
    def company_address(self):
        return self._company_address

    @property
    def city(self):
        return self._city

    @property
    def state(self):
        return self._state

    @property
    def postal_code(self):
        return self._postal_code

    @property
    def deleted_at(self):
        return self._deleted_at

    @property
    def vat(self):
        return self._vat

    @property
    def image(self):
        return self._image

    @property
    def archived_storage(self):
        return self._archived_storage

    @autowire(argmap={"account_id": "id"})
    def add_users_to_teams(
        self, *, account_id=None, role=UserRole.MEMBER, message=None, team_ids=None, user_emails=None, user=None
    ):
        request = (
            HttpRequest.request(Config.api["v1"]["hostname"])
            .post(Config.api["v1"]["endpoints"]["add_users_to_teams"])
            .add_path_param("account_id", account_id)
            .set_header("Authorization", user.session_id)
            .add_body_param("role", role.value)
            .add_body_param("message", message)
        )
        print("adding user to teams in account.py")
       

        if not isinstance(team_ids, list):
            team_ids = [team_ids]
        for idx, team_id in enumerate(team_ids):
            request.add_body_param(f"teams[{idx}][id]", team_id)

        if not isinstance(user_emails, list):
            user_emails = [user_emails]
        for idx, user_email in enumerate(user_emails):
            request.add_body_param(f"users[{idx}][email]", user_email)

        response = request.send()
        print(response.body)
        return response.body().value(key="success").is_equal_to(True)

    @autowire(argmap={"account_id": "id"})
    def remove_users_from_team(self, *, account_id=None, team_id=None, user_ids=None, user=None):
        request = (
            HttpRequest.request(Config.api["v1"]["hostname"])
            .post(Config.api["v1"]["endpoints"]["remove_users_from_team"])
            .add_path_param("account_id", account_id)
            .set_header("Authorization", user.session_id)
            .add_body_param(f"team[id]", team_id)
        )

        if not isinstance(user_ids, list):
            user_ids = [user_ids]
        for idx, user_id in enumerate(user_ids):
            request.add_body_param(f"users[{idx}][id]", user_id)

        response = request.send()
        return response.body().value(key="success").is_equal_to(True)

    @autowire(argmap={"account_id": "id"})
    def remove_users_from_account(self, *, account_id=None, user_ids=None, user=None):
        request = (
            HttpRequest.request(Config.api["v1"]["hostname"])
            .post(Config.api["v1"]["endpoints"]["remove_users_from_account"])
            .add_path_param("account_id", account_id)
            .set_header("Authorization", user.session_id)
        )

        if not isinstance(user_ids, list):
            user_ids = [user_ids]
        request.add_body_param(f"user_ids[]", user_ids)

        response = request.send()
        return response.body().value(key="success").is_equal_to(True)

    @autowire(argmap={"account_id": "id"})
    def update_users_role(self, *, account_id=None, role=UserRole.MEMBER, user_ids=None, user=None):
        request = (
            HttpRequest.request(Config.api["v1"]["hostname"])
            .post(Config.api["v1"]["endpoints"]["update_users_role"])
            .add_path_param("account_id", account_id)
            .set_header("Authorization", user.session_id)
            .add_body_param("role", role.value)
        )

        if not isinstance(user_ids, list):
            user_ids = [user_ids]
        for idx, user_id in enumerate(user_ids):
            request.add_body_param(f"users[{idx}][id]", user_id)

        response = request.send()
        return response.body().value(key="success").is_equal_to(True)

    @autowire(argmap={"account_id": "id"})
    def add_team_manager(self, *, account_id=None, team_id=None, user_id=None, user=None):
        response = (
            HttpRequest.request(Config.api["v1"]["hostname"])
            .post(Config.api["v1"]["endpoints"]["add_team_manager"])
            .add_path_param("account_id", account_id)
            .set_header("Authorization", user.session_id)
            .add_body_param(f"team[id]", team_id)
            .add_body_param(f"user[id]", user_id)
            .send()
        )

        return response.body().value(key="success").is_equal_to(True)

    @autowire(argmap={"account_id": "id"})
    def remove_team_manager(self, *, account_id=None, team_id=None, user_id=None, user=None):
        response = (
            HttpRequest.request(Config.api["v1"]["hostname"])
            .post(Config.api["v1"]["endpoints"]["remove_team_manager"])
            .add_path_param("account_id", account_id)
            .set_header("Authorization", user.session_id)
            .add_body_param(f"team[id]", team_id)
            .add_body_param(f"user[id]", user_id)
            .send()
        )

        return response.body().value(key="success").is_equal_to(True)

    @autowire(argmap={"account_id": "id"})
    def get_teams(self, *, account_id=None, user=None):
        response = (
            HttpRequest.request(Config.api["v1"]["hostname"])
            .get(Config.api["v1"]["endpoints"]["get_account_teams"])
            .add_path_param("account_id", account_id)
            .set_header("Authorization", user.session_id)
            .send()
        )
        print(vars(response))
        return response.to_models(model_clazz=TeamModel, key="teams")



from fiopy.config import Config
from fiopy.models.base import BaseModel
from fiopy.models.project import ProjectModel
from fiopy.http.http_request import HttpRequest
from fiopy.annotations.autowire import autowire
from fiopy.models.project_settings import ProjectSettings


class TeamModel(BaseModel):
    def __init__(self, **kwargs):

        super().__init__(**kwargs)

        self._account_id = kwargs.get("account_id")
        self._allocation_data = kwargs.get("allocation_data")
        self._bio = kwargs.get("bio")
        self._created_at = kwargs.get("created_at")
        self._email_branding = kwargs.get("email_branding")
        self._max_collaborator = kwargs.get("max_collaborators")
        self._max_storage = kwargs.get("max_storage")
        self._member_limit = kwargs.get("member_limit")
        self._total_filesize = kwargs.get("total_filesize")
        self._avatar_url = kwargs.get("upload_url")
        self._watermark_text = kwargs.get("watermark")

        super().cleanup_values()

    @property
    def account_id(self):
        return self._account_id

    @property
    def allocation_data(self):
        return self._allocation_data

    @property
    def bio(self):
        return self._bio

    @property
    def created_at(self):
        return self._created_at

    @property
    def email_branding(self):
        return self._email_branding

    @property
    def max_collaborator(self):
        return self._max_collaborator

    @property
    def max_storage(self):
        return self._max_storage

    @property
    def member_limit(self):
        return self._member_limit

    @property
    def total_filesize(self):
        return self._total_filesize

    @property
    def avatar_url(self):
        return self._avatar_url

    @property
    def watermark_text(self):
        return self._watermark_text

    @autowire(argmap={"team_id": "id"})
    def get_projects(self, *, team_id=None, user=None):
        response = (
            HttpRequest.request(Config.api["v1"]["hostname"])
            .get(Config.api["v1"]["endpoints"]["get_team_projects"])
            .set_header("Authorization", user.session_id)
            .add_path_param("team_id", team_id)
            .send()
        )

        return response.to_models(model_clazz=ProjectModel, key="projects")

    @autowire(argmap={"team_id": "id"})
    def create_project(self, *, team_id: str = None, settings: ProjectSettings = None, user=None):
        response = (
            HttpRequest.request(Config.api["v1"]["hostname"])
            .post(Config.api["v1"]["endpoints"]["create_project"])
            .set_header("Authorization", user.session_id)
            .add_path_param("team_id", team_id)
            .body(settings.settings)
            .send()
        )

        return response.to_model(model_clazz=ProjectModel, key="project")



from fiopy.models.base import BaseModel

__all__ = ["PlanFeaturesModel"]


class PlanFeaturesModel(BaseModel):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self._custom_branded_emails = kwargs.get("custom_branded_emails")
        self._custom_branded_presentations = kwargs.get("custom_branded_presentations")
        self._reel_player = kwargs.get("reel_player")
        self._team_only_comments = kwargs.get("team_only_comments")
        self._archival_storage = kwargs.get("archival_storage")
        self._session_based_watermarking = kwargs.get("session_based_watermarking") 
        self._secure_sharing = kwargs.get("secure_sharing")
        super().cleanup_values()

    @property
    def custom_branded_emails(self):
        return self._custom_branded_emails

    @property
    def custom_branded_presentations(self):
        return self._custom_branded_presentations

    @property
    def reel_player(self):
        return self._reel_player

    @property
    def team_only_comments(self):
        return self._team_only_comments

    @property
    def archival_storage(self):
        return self._archival_storage
 
    @property
    def session_based_watermarking(self):
        return self._session_based_watermarking

    @property
    def secure_sharing(self):
        return self._secure_sharing



from fiopy.models.base_container import BaseContainerModel


class BaseAssetModel(BaseContainerModel):
    def __init__(self, **kwargs):

        super().__init__(**kwargs)

        self._archived_at = kwargs.get("archived_at")
        self._autoversion_id = kwargs.get("autoversion_id")
        self._comment_count = kwargs.get("comment_count")
        self._copy = kwargs.get("copy")

        self._deleted_at = kwargs.get("deleted_at")
        self._description = kwargs.get("description")

        self._duration = kwargs.get("duration")
        self._filesize = kwargs.get("filesize")
        self._filetype = kwargs.get("filetype")
        self._fps = kwargs.get("fps")
        self._frames = kwargs.get("frames")

        self._is_360 = kwargs.get("is_360")
        self._label = kwargs.get("label")

        self._private = kwargs.get("private")
        self._properties = kwargs.get("properties")
        self._rating = kwargs.get("rating")

    @property
    def archived_at(self):
        return self._archived_at

    @property
    def autoversion_id(self):
        return self._autoversion_id

    @property
    def comment_count(self):
        return self._comment_count

    @property
    def copy(self):
        return self._copy

    @property
    def deleted_at(self):
        return self._deleted_at

    @property
    def description(self):
        return self._description

    @property
    def duration(self):
        return self._duration

    @property
    def filesize(self):
        return self._filesize

    @property
    def filetype(self):
        return self._filetype

    @property
    def fps(self):
        return self._fps

    @property
    def frames(self):
        return self._frames

    @property
    def is_360(self):
        return self._is_360

    @property
    def label(self):
        return self._label

    @property
    def private(self):
        return self._private

    @property
    def properties(self):
        return self._properties

    @property
    def rating(self):
        return self._rating




from fiopy.enums import BaseEnum

__all__ = ["UserRole"]


class UserRole(BaseEnum):
    OWNER = "owner"
    ADMIN = "admin"
    MANAGER = "manager"
    MEMBER = "member"
    COLLABORATOR = "collaborator"



class RateLimit:
    def __init__(self, *, value=0, remaining=0, window=0):
        self._value = value
        self._remaining = remaining
        self._window = window

    @property
    def value(self):
        return self._value

    @property
    def remaining(self):
        return self._remaining

    @property
    def window(self):
        return self._window



from fiopy.http.rate_limit import RateLimit


class RateLimitedException(Exception):
    def __init__(self, message: str = None, rate_limit: RateLimit = None):
        super().__init__(message)
        self._rate_limit = rate_limit

    def __str__(self):
        return f"Rate-limited. {self.remaining} requests (out of {self.value} requests) remaining against {self.window} ms."

    @property
    def value(self):
        return self._rate_limit.value

    @property
    def remaining(self):
        return self._rate_limit.remaining

    def window(self):
        return self._rate_limit.window



__all__ = ["BaseModel"]

class BaseModel:
    def __init__(self, **kwargs):
        self._id = kwargs.get("id")

    def __eq__(self, other):
        return self.__dict__ == other.__dict__

    def __bool__(self):
        return bool(self.id)

    def __str__(self):
        return self.id

    def is_not_none(self):
        return self._id is not None

    @property
    def id(self):
        return self._id

    @id.setter
    def id(self, id):
        self._id = id

    def cleanup_values(self):
        for key, value in self.__dict__.items():
            if not isinstance(value, BaseModel) and value in ["JTNullValue", "null"]:
                self.__dict__[key] = None




from fiopy.config import Config
from fiopy.enums import SessionMethod, UserRole
from fiopy.models.account import AccountModel
from fiopy.models.base import BaseModel
from fiopy.models.team import TeamModel
from fiopy.models.project import ProjectModel
from fiopy.http.http_request import HttpRequest
from fiopy.annotations.autowire import autowire


class UserModel(BaseModel):
    def __init__(self, **kwargs):

        super().__init__(**kwargs)

        self._email = kwargs.get("email")
        self._password = kwargs.get("password")
        self._organization = kwargs.get("organization")
        try:
            self._role = UserRole(kwargs.get("role"))
        except ValueError:
            self._role = None
        self._account_id = kwargs.get("account_id")
        self._session_jwt = kwargs.get("session_jwt")
        self._session_pat = kwargs.get("session_pat")
        self._session_id = self._session_jwt if self._session_jwt else self._session_pat
        self._session_method = SessionMethod.JWT if self._session_jwt else SessionMethod.PAT
        self._bio = None
        self._deleted_at = None
        self._digest_frequency = None
        self._email_confirm_by = None
        self._email_preferences = None
        self._features_seen = None
        self._first_login_at = None
        self._from_google = None
        self._image_128 = None
        self._image_256 = None
        self._image_32 = None
        self._image_64 = None
        self._inserted_at = None
        self._joined_via = None
        self._last_seen = None
        self._location = None
        self._name = None
        self._next_digest_date = None
        self._phone = None
        self._profile_image = None
        self._profile_image_original = None
        self._roles = None
        self._timezone_value = None
        self._updated_at = None
        self._upload_url = None
        self._user_default_color = None
        self._user_hash = None

        self.get_me()

    @property
    def email(self):
        return self._email

    @property
    def password(self):
        return self._password

    @property
    def organization(self):
        return self._organization

    @organization.setter
    def organization(self, org):
        self.organization = org

    @property
    def role(self):
        return self._role

    @role.setter
    def role(self, ro):
        self._role = UserRole(ro)

    @property
    def account_id(self):
        return self._account_id

    @property
    def session_jwt(self):
        return self._session_jwt

    @property
    def session_pat(self):
        return self._session_pat

    @property
    def session_id(self):
        return self._session_id

    @property
    def session_method(self):
        return self._session_method

    @session_method.setter
    def session_method(self, method):
        self._session_id = getattr(self, method.value)
        self._session_method = method

    @property
    def from_google(self):
        return self._from_google

    @property
    def image_128(self):
        return self._image_128

    @property
    def image_256(self):
        return self._image_256

    @property
    def image_32(self):
        return self._image_32

    @property
    def image_64(self):
        return self._image_64

    @property
    def inserted_at(self):
        return self._inserted_at

    @property
    def joined_via(self):
        return self._joined_via

    @property
    def last_seen(self):
        return self._last_seen

    @property
    def location(self):
        return self._location

    @property
    def name(self):
        return self._name

    @property
    def next_digest_date(self):
        return self._next_digest_date

    @property
    def phone(self):
        return self._phone

    @property
    def profile_image(self):
        return self._profile_image

    @property
    def profile_image_original(self):
        return self._profile_image_original

    @property
    def roles(self):
        return self._roles

    @property
    def timezone_value(self):
        return self._timezone_value

    @property
    def updated_at(self):
        return self._updated_at

    @property
    def upload_url(self):
        return self._upload_url

    @property
    def user_default_color(self):
        return self._user_default_color

    @property
    def user_hash(self):
        return self._user_hash

    def get_me(self):
        response = (
            HttpRequest.request(Config.api["v2"]["hostname"])
            .get(Config.api["v2"]["endpoints"]["get_me"])
            .set_header("Authorization", self.session_id)
            .send()
        )

        body = response.body().get()
        print(body)
        self._id = body["id"]
        self._account_id = body["account_id"]
        self._bio = body["bio"]
        self._deleted_at = body["deleted_at"]
        self._digest_frequency = body["digest_frequency"]
        self._email_confirm_by = body["email_confirm_by"]
        self._email_preferences = body["email_preferences"]
        self._features_seen = body["features_seen"]
        self._first_login_at = body["first_login_at"]
        self._from_google = body["from_google"]
        self._image_128 = body["image_128"]
        self._image_256 = body["image_256"]
        self._image_32 = body["image_32"]
        self._image_64 = body["image_64"]
        self._inserted_at = body["inserted_at"]
        self._joined_via = body["joined_via"]
        self._last_seen = body["last_seen"]
        self._location = body["location"]
        self._name = body["name"]
        self._next_digest_date = body["next_digest_date"]
        self._phone = body["phone"]
        self._profile_image = body["profile_image"]
        self._profile_image_original = body["profile_image_original"]
        self._roles = body["roles"]
        self._timezone_value = body["timezone_value"]
        self._updated_at = body["updated_at"]
        self._upload_url = body["upload_url"]
        self._user_default_color = body["user_default_color"]
        self._user_hash = body["user_hash"]

        super().cleanup_values()

        return self

    @autowire()
    def get_accounts(self, *, user=None):
        response = (
            HttpRequest.request(Config.api["v2"]["hostname"])
            .get(Config.api["v2"]["endpoints"]["get_accounts"])
            .set_header("Authorization", user.session_id)
            .send()
        )

        return response.to_models(model_clazz=AccountModel)

    @autowire(argmap={"account_id": "account_id"})
    def get_account(self, *, account_id=None, user=None):
        response = (
            HttpRequest.request(Config.api["v2"]["hostname"])
            .get(Config.api["v2"]["endpoints"]["get_account"])
            .add_path_param("account_id", account_id)
            .set_header("Authorization", user.session_id)
            .send()
        )

        return response.to_model(model_clazz=AccountModel)

    @autowire(argmap={"user_id": "id"})
    def get_teams(self, *, user_id, user=None):
        response = (
            HttpRequest.request(Config.api["v1"]["hostname"])
            .get(Config.api["v1"]["endpoints"]["get_user_teams"])
            .add_path_param("user_id", user_id)
            .set_header("Authorization", user.session_id)
            .send()
        )
        print(response.body.get())
        return response.to_models(model_clazz=TeamModel, key="teams")

    @autowire()
    def get_team(self, *, team_id=None, user=None):
        response = (
            HttpRequest.request(Config.api["v1"]["hostname"])
            .get(Config.api["v1"]["endpoints"]["get_team"])
            .add_path_param("team_id", team_id)
            .set_header("Authorization", user.session_id)
            .send()
        )

        return response.to_model(model_clazz=TeamModel)

    @autowire()
    def get_project(self, *, project_id=None, user=None):
        response = (
            HttpRequest.request(Config.api["v1"]["hostname"])
            .get(Config.api["v2"]["endpoints"]["get_project"])
            .add_path_param("project_id", project_id)
            .set_header("Authorization", user.session_id)
            .send()
        )

        return response.to_model(model_clazz=ProjectModel)



from __future__ import annotations

import json
from typing import TYPE_CHECKING

from fiopy.annotations.autowire import autowire
from fiopy.config import Config
from fiopy.enums import UserRole
from fiopy.http.http_request import HttpRequest
from fiopy.models.base import BaseModel
from fiopy.models.project_prefs import ProjectPrefsModel
from fiopy.models.project_root_folder import ProjectRootFolderModel
from fiopy.models.project_user_prefs import ProjectUserPrefsModel

if TYPE_CHECKING:
    from fiopy.models.user import UserModel


class ProjectModel(BaseModel):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        self._name = kwargs.get("name")
        self._description = kwargs.get("description")

        self._size = kwargs.get("size")

        self._team_id = kwargs.get("team_id")
        self._user_id = kwargs.get("user_id")
        self._root_folder_key = kwargs.get("root_folder_key")
        self._root_folder = None
        self._invite_url = kwargs.get("invite_url")

        self._archived_at = kwargs.get("archived_at")
        self._clear_from_usage = kwargs.get("clear_from_usage")
        self._ignore_archive = kwargs.get("ignore_archive")
        self._joinable = kwargs.get("joinable")
        self._private = kwargs.get("private")
        project_prefs = kwargs.get("project_prefs", {})
        while isinstance(project_prefs, str):
            project_prefs = json.loads(project_prefs)
        self._project_prefs = ProjectPrefsModel(**project_prefs)
        self._shared = kwargs.get("shared")
        self._slack_notifications_pref = kwargs.get("slack_notifications_pref")

        user_prefs = kwargs.get("user_prefs", {})
        while isinstance(user_prefs, str):
            user_prefs = json.loads(user_prefs)
        self._user_prefs = ProjectUserPrefsModel(**user_prefs)
        self._view_downloadable = kwargs.get("view_downloadable")
        self._view_only = kwargs.get("view_only")

        self._created_at_integer = kwargs.get("created_at_integer")
        self._updated_at_integer = kwargs.get("updated_at_integer")

        super().cleanup_values()

    @property
    def name(self):
        return self._name

    @property
    def description(self):
        return self._description

    @property
    def size(self):
        return self._size

    @property
    def team_id(self):
        return self._team_id

    @property
    def user_id(self):
        return self._user_id

    @property
    def root_folder_key(self):
        return self._root_folder_key

    @property
    def root_folder(self):
        if not self._root_folder:
            self._root_folder = self._get_root_folder()
        return self._root_folder

    @root_folder.setter
    def root_folder(self, value):
        self._root_folder = value

    @property
    def invite_url(self):
        return self._invite_url

    @property
    def archived_at(self):
        return self._archived_at

    @property
    def clear_from_usage(self):
        return self._clear_from_usage

    @property
    def ignore_archive(self):
        return self._ignore_archive

    @property
    def joinable(self):
        return self._joinable

    @property
    def private(self):
        return self._private

    @property
    def project_prefs(self):
        return self._project_prefs

    @property
    def shared(self):
        return self._shared

    @property
    def slack_notifications_pref(self):
        return self._slack_notifications_pref

    @property
    def user_prefs(self):
        return self._user_prefs

    @property
    def view_downloadable(self):
        return self._view_downloadable

    @property
    def view_only(self):
        return self._view_only

    @property
    def created_at_integer(self):
        return self._created_at_integer

    @property
    def updated_at_integer(self):
        return self._updated_at_integer

    @autowire(argmap={"root_folder_id": "root_folder_key"})
    def _get_root_folder(self, root_folder_id: str = None, user: UserModel = None):
        response = (
            HttpRequest.request(Config.api["v2"]["hostname"])
            .get(Config.api["v2"]["endpoints"]["get_asset"])
            .add_path_param("asset_id", root_folder_id)
            .set_header("Authorization", user.session_id)
            .send()
        )

        return response.to_model(model_clazz=ProjectRootFolderModel)

    def join(self, *, project_id: str = None, user: UserModel = None):
        response = (
            HttpRequest.request(Config.api["v1"]["hostname"])
            .get(Config.api["join_project"])
            .add_path_param("project_id", project_id)
            .set_header("Authorization", user.session_id)
            .send()
        )

        return response

    @autowire(argmap={"project_id": "id"})
    def leave(self, *, user_ids: list = None, project_id: str = None, user: UserModel = None):
        user_ids = user_ids or [user.id]
        response = (
            HttpRequest.request(Config.api["v1"]["hostname"])
            .post(Config.api["join_project"])
            .add_path_param("project_id", project_id)
            .set_header("Authorization", user.session_id)
            .body({"collaboratorIds": user_ids})
            .send()
        )

        return response

    @autowire(argmap={"project_id": "id"})
    def delete(self, *, project_id: str = None, user: UserModel = None):
        response = (
            HttpRequest.request(Config.api["v1"]["hostname"])
            .post(Config.api["v1"]["endpoints"]["delete_project"])
            .add_path_param("project_id", project_id)
            .set_header("Authorization", user.session_id)
            .send()
        )

        return response.body().value(key="success").is_equal_to(True)

    def upload(self, *, file_path: str = None, folder_path: str = None, user: UserModel = None):
        return self.root_folder.upload(file_path=file_path, folder_path=folder_path, user=user)

    def get_children(self, *, asset_id: str = None, user: UserModel = None):
        return self.root_folder.get_children(asset_id=asset_id, user=user)




from fiopy.config import Config
from fiopy.enums import ConfigType
from fiopy.models.user import UserModel
from fiopy.http.http_request import HttpRequest

__all__ = ["Fio"]


class Fio:
    @staticmethod
    def login(*, email: str = None, password: str = None, token: str = None, **kwargs):
        user_data = {}
        user_data.update(kwargs)
        print("AM I HEREREERE")
        if email and password:
            response = (
                HttpRequest(Config.api["v2"]["hostname"])
                .post(Config.api["v2"]["endpoints"]["login"])
                .body({"email": email, "password": password})
                .send()
            )
            print(response.body)
            if response.status().is_equal_to(200):
                session_jwt = response.header("authorization").get()
                user_data.update(response.body().get())
                user_data.update({"session_jwt": session_jwt})
        else:
            session_pat = token
            user_data.update({"session_pat": session_pat})
        print("logging with FIO")
        model = UserModel(**user_data)
        Fio._set_default_user(model)
        return model

    @staticmethod
    def _set_default_user(user: UserModel = None):
        Config.update_config(config=user, config_type=ConfigType.USER)

    @staticmethod
    def update_config(config, config_type):
        Config.update_config(config=config, config_type=config_type)

    @staticmethod
    def default_user():
        return Config.user



from __future__ import annotations

from typing import TYPE_CHECKING

from fiopy.config import Config
from fiopy.models.base import BaseModel
from fiopy.http.http_request import HttpRequest
from fiopy.annotations.autowire import autowire

if TYPE_CHECKING:
    from fiopy.models.user import UserModel


class BaseContainerModel(BaseModel):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self._item_count = kwargs.get("item_count")
        self._name = kwargs.get("name")
        self._created_at = kwargs.get("created_at_integer") or kwargs.get("inserted_at")
        self._updated_at = kwargs.get("updated_at_integer") or kwargs.get("updated_at")
        self._parent_id = kwargs.get("parent_id") or kwargs.get("owner_key")
        self._project_id = kwargs.get("project_id") or kwargs.get("project_key")
        self._private = kwargs.get("private", False)

    @property
    def item_count(self):
        return self._item_count

    @property
    def name(self):
        return self._name

    @property
    def created_at(self):
        return self._created_at

    @property
    def updated_at(self):
        return self._updated_at

    @property
    def parent_id(self):
        return self._parent_id

    @property
    def project_id(self):
        return self._project_id

    @autowire(argmap={"asset_id": "id"})
    def get_children(self, *, asset_id: str = None, user: UserModel = None):
        response = (
            HttpRequest.request(Config.api["v2"]["hostname"])
            .get(Config.api["v2"]["endpoints"]["get_asset_children"])
            .add_path_param("asset_id", asset_id)
            .set_header("Authorization", user.session_id)
            .send()
        )
        from fiopy.models.asset import AssetModel

        return response.to_models(model_clazz=AssetModel)




from fiopy.config import Config
from fiopy.enums import UserRole
from fiopy.models.base import BaseModel
from fiopy.http.http_request import HttpRequest
from fiopy.annotations.autowire import autowire

__all__ = ["ProjectPrefsModel"]


class ProjectPrefsModel(BaseModel):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        super().cleanup_values()




from fiopy.models.base import BaseModel
from fiopy.models.comment import CommentModel


class ReviewLinkItemModel(BaseModel):
    def __init__(self, **kwargs):

        super().__init__(**kwargs)

        self._review_link_id = kwargs.get("review_link_id")
        self._asset_id = kwargs.get("asset_id")
        self._index = kwargs.get("index")

        self._inserted_at = kwargs.get("inserted_at")
        self._deleted_at = kwargs.get("deleted_at")
        self._updated_at = kwargs.get("updated_at")

        super().cleanup_values()

    @property
    def review_link_id(self):
        return self._review_link_id

    @property
    def asset_id(self):
        return self._asset_id

    @property
    def index(self):
        return self._index

    @property
    def inserted_at(self):
        return self._inserted_at

    @property
    def deleted_at(self):
        return self._deleted_at

    @property
    def updated_at(self):
        return self._updated_at




from fiopy.enums.base import BaseEnum
from fiopy.enums.config_type import ConfigType
from fiopy.enums.session_method import SessionMethod
from fiopy.enums.user_role import UserRole

__all__ = ["UserRole", "SessionMethod", "ConfigType"]



from fiopy.models.base import BaseModel


__all__ = ["PromotionModel"]


class PromotionModel(BaseModel):
    def __init__(self, **kwargs):

        super().__init__(**kwargs)
        self._plan_id = kwargs.get("plan_id")

        self._header_text = kwargs.get("header_text")
        self._header_subtext = kwargs.get("header_subtext")

        self._inserted_at = kwargs.get("inserted_at")
        self._deleted_at = kwargs.get("deleted_at")
        self._expires_at = kwargs.get("expires_at")
        self._updated_at = kwargs.get("updated_at")

        self._is_trial = kwargs.get("is_trial")

        self._autoscaling = kwargs.get("autoscaling")
        self._can_override_limitations = kwargs.get("can_override_limitations")
        self._new_price = kwargs.get("new_price")
        self._no_credit_card = kwargs.get("no_credit_card")
        self._promo_code = kwargs.get("promo_code")
        self._submit_text = kwargs.get("submit_text")
        self._trial_length = kwargs.get("trial_length")

        super().cleanup_values()

    @property
    def plan_id(self):
        return self._plan_id

    @property
    def header_text(self):
        return self._header_text

    @property
    def header_subtext(self):
        return self._header_subtext

    @property
    def inserted_at(self):
        return self._inserted_at

    @property
    def deleted_at(self):
        return self._deleted_at

    @property
    def expires_at(self):
        return self._expires_at

    @property
    def updated_at(self):
        return self._updated_at

    @property
    def is_trial(self):
        return self._is_trial

    @property
    def autoscaling(self):
        return self._autoscaling

    @property
    def can_override_limitations(self):
        return self._can_override_limitations

    @property
    def new_price(self):
        return self._new_price

    @property
    def no_credit_card(self):
        return self._no_credit_card

    @property
    def promo_code(self):
        return self._promo_code

    @property
    def submit_text(self):
        return self._submit_text

    @property
    def trial_length(self):
        return self._trial_length




import json
from fiopy.models.base import BaseModel
from fiopy.models.plan_features import PlanFeaturesModel

__all__ = ["PlanModel"]


class PlanModel(BaseModel):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self._cost = kwargs.get("cost")

        self._name = kwargs.get("name")
        self._title = kwargs.get("title")
        self._payment_method = kwargs.get("payment_method")
        self._period = kwargs.get("period")
        self._enterprise = kwargs.get("enterprise")
        self._default_plan = kwargs.get("default_plan")
        self._tier = kwargs.get("tier")

        self._member_limit = kwargs.get("member_limit")
        self._collaborator_limit = kwargs.get("collaborator_limit")
        self._autoscaling = kwargs.get("autoscaling")
        self._archived_storage_limit = kwargs.get("archived_storage_limit")
        self._file_limit = kwargs.get("file_limit")
        self._lifetime_file_limit = kwargs.get("lifetime_file_limit")
        self._project_limit = kwargs.get("project_limit")
        self._storage_limit = kwargs.get("storage_limit")
        self._team_limit = kwargs.get("team_limit")

        available_features = kwargs.get("available_features", {})
        while isinstance(available_features, str):
            available_features = json.loads(available_features)
        self._available_features = PlanFeaturesModel(**available_features)

        self._updated_at = kwargs.get("updated_at")
        self._version = kwargs.get("version")

        super().cleanup_values()

    @property
    def cost(self):
        return self._cost

    @property
    def name(self):
        return self._name

    @property
    def title(self):
        return self._title

    @property
    def payment_method(self):
        return self._payment_method

    @property
    def period(self):
        return self._period

    @property
    def enterprise(self):
        return self._enterprise

    @property
    def default_plan(self):
        return self._default_plan

    @property
    def tier(self):
        return self._tier

    @property
    def member_limit(self):
        return self._member_limit

    @property
    def collaborator_limit(self):
        return self._collaborator_limit

    @property
    def autoscaling(self):
        return self._autoscaling

    @property
    def archived_storage_limit(self):
        return self._archived_storage_limit

    @property
    def file_limit(self):
        return self._file_limit

    @property
    def lifetime_file_limit(self):
        return self._lifetime_file_limit

    @property
    def project_limit(self):
        return self._project_limit

    @property
    def storage_limit(self):
        return self._storage_limit

    @property
    def team_limit(self):
        return self._team_limit

    @property
    def available_features(self):
        return self._available_features

    @property
    def updated_at(self):
        return self._updated_at

    @property
    def version(self):
        return self._version




from typing import TYPE_CHECKING

from fiopy.config import Config
from fiopy.models.base import BaseModel
from fiopy.annotations.autowire import autowire
from fiopy.http.http_request import HttpRequest
from fiopy.models.asset_impression import AssetImpression

if TYPE_CHECKING:
    from fiopy.models.user import UserModel


class ReviewLinkModel(BaseModel):
    def __init__(self, **kwargs):

        super().__init__(**kwargs)

        self._allow_approvals = kwargs.get("allow_approvals")
        self._collaborators_and_team_members_ids = kwargs.get("collaborators_and_team_members_ids")
        self._created_at = kwargs.get("created_at")
        self._creator_key = kwargs.get("creator_key")
        self._current_version_only = kwargs.get("current_version_only")
        self._enable_downloading = kwargs.get("enable_downloading")
        self._expires_at = kwargs.get("expires_at")
        self._file_reference_keys = kwargs.get("file_reference_keys")
        self._file_references = kwargs.get("file_references")
        self._has_password = kwargs.get("has_password")
        self._is_active = kwargs.get("is_active")
        self._item_count = kwargs.get("item_count")
        self._master = kwargs.get("master")
        self._master_class = kwargs.get("master_class")
        self._name = kwargs.get("name")
        self._notify_on_view = kwargs.get("notify_on_view")
        self._owner = kwargs.get("owner")
        self._owner_id = kwargs.get("owner_id")
        self._password = kwargs.get("password")
        self._project_id = kwargs.get("project_key")
        self._account_id = kwargs.get("account_key")
        self._team_id = kwargs.get("team", {}).get("id")

        self._updated_at = kwargs.get("updated_at")
        self._user = kwargs.get("user")
        self._version_stack_keys = kwargs.get("version_stack_keys")
        self._version_stacks = kwargs.get("version_stacks")
        self._views_count = kwargs.get("views_count")

        super().cleanup_values()

    @property
    def allow_approvals(self):
        return self._allow_approvals

    @property
    def collaborators_and_team_members_ids(self):
        return self._collaborators_and_team_members_ids

    @property
    def created_at(self):
        return self._created_at

    @property
    def creator_key(self):
        return self._creator_key

    @property
    def current_version_only(self):
        return self._current_version_only

    @property
    def enable_downloading(self):
        return self._enable_downloading

    @property
    def expires_at(self):
        return self._expires_at

    @property
    def file_reference_keys(self):
        return self._file_reference_keys

    @property
    def file_references(self):
        return self._file_references

    @property
    def has_password(self):
        return self._has_password

    @property
    def is_active(self):
        return self._is_active

    @property
    def item_count(self):
        return self._item_count

    @property
    def master(self):
        return self._master

    @property
    def master_class(self):
        return self._master_class

    @property
    def name(self):
        return self._name

    @property
    def notify_on_view(self):
        return self._notify_on_view

    @property
    def owner(self):
        return self._owner

    @property
    def owner_id(self):
        return self._owner_id

    @property
    def password(self):
        return self._password

    @property
    def project_id(self):
        return self._project_id

    @property
    def account_id(self):
        return self._account_id

    @property
    def team_id(self):
        return self._team_id

    @property
    def updated_at(self):
        return self._updated_at

    @property
    def user(self):
        return self._user

    @property
    def version_stack_keys(self):
        return self._version_stack_keys

    @property
    def version_stacks(self):
        return self._version_stacks

    @property
    def views_count(self):
        return self._views_count

    @autowire(argmap={"review_link_id": "id"})
    def get_review_link_items(self, review_link_id: str = None, user: UserModel = None) -> list:
        response = (
            HttpRequest.request(Config.api["v2"]["hostname"])
            .get(Config.api["v2"]["endpoints"]["get_review_link_items"])
            .add_path_param("review_link_id", review_link_id)
            .set_header("Authorization", user.session_id)
            .send()
        )

        return response.to_models(model_clazz=ReviewLinkModel)




from fiopy.http.rate_limit import RateLimit
from fiopy.models.base import BaseModel
import jmespath
import os


class HttpResponse:
    def __init__(self, res):
        self._status = res.status_code
        self._headers = res.headers
        try:
            self._body = res.json()
        except (ValueError, UnicodeDecodeError):
            self._body = res.text
        self._rate_limit = RateLimit(
            value=res.headers.get("x-ratelimit-limit"),
            remaining=res.headers.get("x-ratelimit-remaining"),
            window=res.headers.get("x-ratelimit-window"),
        )

        if os.getenv("FIO_DEBUG"):
            print(
                "{url}\n\n{status_code}\n{headers}\n\n{body}".format(
                    url=res.url,
                    status_code=res.status_code,
                    headers="\n".join("{}: {}".format(k, v) for k, v in res.headers.items()),
                    body=res.content,
                )
            )

    def status(self):
        self._assertion_value = self._status
        return self

    def body(self):
        self._assertion_value = self._body
        return self

    def header(self, name):
        self._assertion_value = self._headers[name]
        return self

    def to_model(self, *, model_clazz: BaseModel = None, key: str = None, jpath: str = None) -> BaseModel:
        data = self.body().value(key=key, jpath=jpath).get() if key or jpath else self.body().get()
        model = model_clazz(**(data or {}))
        return model

    def to_models(self, *, model_clazz: BaseModel = None, key: str = None, jpath: str = None) -> BaseModel:
        data = self.body().value(key=key, jpath=jpath).get() if key or jpath else self.body().get()

        models = []
        if isinstance(data, list) and data:
            models = list(filter(bool, [model_clazz(**(obj or {})) for obj in data]))
        return models

    @property
    def rate_limit(self):
        return self._rate_limit

    def get(self):
        assertion_result = self._assertion_value
        self._assertion_value = None
        return assertion_result

    def value(self, *, key=None, jpath=None):
        if jpath:
            self._assertion_value = jmespath.search(jpath, self._assertion_value)
        elif key:
            self._assertion_value = self._assertion_value.get(key)
        return self

    def is_equal_to(self, value):
        assertion_result = self._assertion_value == value
        self._assertion_value = None
        return assertion_result

    def is_not_equal_to(self, value):
        assertion_result = self._assertion_value != value
        self._assertion_value = None
        return assertion_result

    def is_of_type(self, type):
        assertion_result = isinstance(self._assertion_value, type)
        self._assertion_value = None
        return assertion_result

    def is_true(self):
        assertion_result = self._assertion_value is True
        self._assertion_value = None
        return assertion_result

    def is_false(self):
        assertion_result = self._assertion_value is False
        self._assertion_value = None
        return assertion_result

    def exists(self):
        assertion_result = self._assertion_value != None
        self._assertion_value = None
        return assertion_result

    def doesnt_exist(self):
        assertion_result = self._assertion_value is None
        self._assertion_value = None
        return assertion_result

    def is_empty(self):
        assertion_result = not bool(self._assertion_value)
        self._assertion_value = None
        return assertion_result

    def is_not_empty(self):
        assertion_result = bool(self._assertion_value)
        self._assertion_value = None
        return assertion_result

    def has_length(self, length):
        assertion_result = len(self._assertion_value) is length
        self._assertion_value = None
        return assertion_result

    def has_length_less_than(self, length):
        assertion_result = len(self._assertion_value) < length
        self._assertion_value = None
        return assertion_result

    def has_length_less_than_or_equal_to(self, length):
        assertion_result = len(self._assertion_value) <= length
        self._assertion_value = None
        return assertion_result

    def has_length_greater_than(self, length):
        assertion_result = len(self._assertion_value) > length
        self._assertion_value = None
        return assertion_result

    def has_length_greater_than_or_equal_to(self, length):
        assertion_result = len(self._assertion_value) >= length
        self._assertion_value = None
        return assertion_result



import json
import os
import requests
from fiopy.http.enums.http_request_method import HttpRequestMethod
from fiopy.http.http_response import HttpResponse
from fiopy.exceptions.rate_limited import RateLimitedException


class HttpRequest:
    @staticmethod
    def request(hostname):
        return HttpRequest(hostname)

    def __init__(self, hostname):
        self._hostname = hostname
        self._endpoint = None
        self._method = None
        self._headers = {}
        self._qs_params = {}
        self._body_params = {}
        self._body = None
        self._file_path = None
        self._body_type = None

    def get(self, endpoint):
        self._endpoint = endpoint
        self._method = HttpRequestMethod.GET
        return self

    def post(self, endpoint):
        self._endpoint = endpoint
        self._method = HttpRequestMethod.POST
        return self

    def put(self, endpoint):
        self._endpoint = endpoint
        self._method = HttpRequestMethod.PUT
        return self

    def delete(self, endpoint):
        self._endpoint = endpoint
        self._method = HttpRequestMethod.DELETE
        return self

    def set_header(self, name, value):
        self._headers[name] = value
        return self

    def add_path_param(self, name, value):
        self._endpoint = self._endpoint.replace(f"${{{name}}}", value, 1)
        return self

    def add_qry_param(self, name, value):
        self._qs_params[name] = value
        return self

    def add_body_param(self, name, value):
        self._body_params[name] = value
        return self

    def body(self, payload):
        try:
            self._body = json.dumps(payload.__dict__)
            self.set_header("content-type", "application/json;charset=UTF-8")
        except:
            try:
                self._body = json.dumps(payload)
                self.set_header("content-type", "application/json;charset=UTF-8")
            except:
                self._body = payload

        return self

    def attach(self, payload):
        self._body = payload
        return self

    def upload(self, file_path):
        self._file_path = file_path
        return self

    def send(self):
        res = requests.request(
            self._method.value,
            f"{self._hostname}{self._endpoint}",
            params=self._qs_params,
            data=self._body or self._body_params,
            json=self._body,
            headers=self._headers,
            proxies={"http": os.getenv("FIO_HTTP_PROXY"), "https": os.getenv("FIO_HTTPS_PROXY")},
            verify=False if os.getenv("FIO_DISABLE_CERT_VALIDATION") else True,
        )
        print("Sending Request")
        response = HttpResponse(res)
        HttpRequest._last_response = response

        if res.status_code == 429:
            raise RateLimitedException(rate_limit=response.rate_limit)

        return HttpRequest._last_response

    @staticmethod
    def get_last_response():
        return HttpRequest._last_response



import json

from fiopy.models.base import BaseModel
from fiopy.models.plan import PlanModel
from fiopy.models.promotion import PromotionModel

__all__ = ["SubscriptionModel"]


class SubscriptionModel(BaseModel):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        self._balance = kwargs.get("balance")

        self._inserted_at = kwargs.get("inserted_at")
        self._deleted_at = kwargs.get("deleted_at")
        self._updated_at = kwargs.get("updated_at")

        self._cancellation_option = kwargs.get("cancellation_option")
        self._cancellation_reason = kwargs.get("cancellation_reason")
        self._cancelled_at = kwargs.get("cancelled_at")
        self._subscription_end_at = kwargs.get("subscription_end_at")

        self._on_trial = kwargs.get("on_trial")
        self._stripe_customer_id = kwargs.get("stripe_customer_id")
        self._stripe_subscription_id = kwargs.get("stripe_subscription_id")
        self._next_bill_at = kwargs.get("next_bill_at")
        self._last_payment_at = kwargs.get("last_payment_at")

        plan = kwargs.get("plan", {})
        while isinstance(plan, str):
            plan = json.loads(plan)
        self._plan = PlanModel(**plan)

        promotion = kwargs.get("promotion", {})
        while isinstance(promotion, str):
            promotion = json.loads(promotion)
        self._promotion = PromotionModel(**promotion)

        super().cleanup_values()

    @property
    def balance(self):
        return self._balance

    @property
    def inserted_at(self):
        return self._inserted_at

    @property
    def deleted_at(self):
        return self._deleted_at

    @property
    def updated_at(self):
        return self._updated_at

    @property
    def cancellation_option(self):
        return self._cancellation_option

    @property
    def cancellation_reason(self):
        return self._cancellation_reason

    @property
    def cancelled_at(self):
        return self._cancelled_at

    @property
    def subscription_end_at(self):
        return self._subscription_end_at

    @property
    def on_trial(self):
        return self._on_trial

    @property
    def stripe_customer_id(self):
        return self._stripe_customer_id

    @property
    def stripe_subscription_id(self):
        return self._stripe_subscription_id

    @property
    def next_bill_at(self):
        return self._next_bill_at

    @property
    def last_payment_at(self):
        return self._last_payment_at

    @property
    def plan(self):
        return self._plan

    @property
    def promotion(self):
        return self._promotion




from setuptools import find_packages, setup

setup(
    name="fiopyhelper",
    version="0.0.1",
    description="FrameIO Python Client",
    packages=find_packages(),
    setup_requires=["pytest-runner"],
    tests_require=["pytest", "testfixtures", "responses"],
    install_requires=[
        "requests==2.27.1",
        "jmespath==1.0.0",
        "pyyaml==6.0",
        "python-magic==0.4.25",
        "deepmerge==1.0.1",
    ],
    include_package_data=True,
)



from deepmerge import Merger


class ProjectSettings:
    def __init__(self):
        self._settings = {
            "project": {"name": "", "private": "0", "team_id": "JTNullValue", "shared": "0", "ignore_archive": "0"},
            "project_preferences": {
                "collaborator_can_download": "1",
                "collaborator_can_invite": "1",
                "collaborator_can_share": "1",
                "email_for_new_comment": "1",
                "email_for_new_person": "1",
                "email_for_new_video": "1",
                "notify_on_updated_label": "1",
            },
            "preferences": {
                "email_for_new_comment": "1",
                "email_for_new_person": "1",
                "email_for_new_video": "1",
                "notify_on_updated_label": "1",
            },
            "slack_notifications_pref": "0",
            "private": "0",
        }

    def update(self, *, new_settings):
        merger = Merger([(list, ["append"]), (dict, ["merge"])], ["override"], ["override"])
        merger.merge(self._settings, new_settings)

    @property
    def settings(self):
        return self._settings

    @property
    def name(self):
        return self._settings["project"]["name"]

    @name.setter
    def name(self, value):
        self._settings["project"]["name"] = value

    @property
    def private(self):
        return self._settings["project"]["private"]

    @private.setter
    def private(self, value):
        self._settings["project"]["private"] = "0" if value == False else "1"
        self._settings["preferences"]["private"] = "0" if value == False else "1"

    @property
    def team_id(self):
        return self._settings["project"]["team_id"]

    @team_id.setter
    def team_id(self, value):
        self._settings["project"]["team_id"] = value

    @property
    def shared(self):
        return self._settings["project"]["shared"]

    @shared.setter
    def shared(self, value):
        self._settings["project"]["shared"] = "0" if value == False else "1"

    @property
    def ignore_archive(self):
        return self._settings["project"]["ignore_archive"]

    @ignore_archive.setter
    def ignore_archive(self, value):
        self._settings["project"]["ignore_archive"] = "0" if value == False else "1"

    @property
    def collaborator_can_download(self):
        return self._settings["project_preferences"]["collaborator_can_download"]

    @collaborator_can_download.setter
    def collaborator_can_download(self, value):
        self._settings["project_preferences"]["collaborator_can_download"] = "0" if value == False else "1"

    @property
    def collaborator_can_invite(self):
        return self._settings["project_preferences"]["collaborator_can_invite"]

    @collaborator_can_invite.setter
    def collaborator_can_invite(self, value):
        self._settings["project_preferences"]["collaborator_can_invite"] = "0" if value == False else "1"

    @property
    def collaborator_can_share(self):
        return self._settings["project_preferences"]["collaborator_can_share"]

    @collaborator_can_share.setter
    def collaborator_can_share(self, value):
        self._settings["project_preferences"]["collaborator_can_share"] = "0" if value == False else "1"

    @property
    def email_for_new_comment(self):
        return self._settings["project_preferences"]["email_for_new_comment"]

    @email_for_new_comment.setter
    def email_for_new_comment(self, value):
        self._settings["project_preferences"]["email_for_new_comment"] = "0" if value == False else "1"
        self._settings["preferences"]["email_for_new_comment"] = "0" if value == False else "1"

    @property
    def email_for_new_person(self):
        return self._settings["project_preferences"]["email_for_new_person"]

    @email_for_new_person.setter
    def email_for_new_person(self, value):
        self._settings["project_preferences"]["email_for_new_person"] = "0" if value == False else "1"
        self._settings["preferences"]["email_for_new_person"] = "0" if value == False else "1"

    @property
    def email_for_new_video(self):
        return self._settings["project_preferences"]["email_for_new_video"]

    @email_for_new_video.setter
    def email_for_new_video(self, value):
        self._settings["project_preferences"]["email_for_new_video"] = "0" if value == False else "1"
        self._settings["preferences"]["email_for_new_video"] = "0" if value == False else "1"

    @property
    def notify_on_updated_label(self):
        return self._settings["project_preferences"]["notify_on_updated_label"]

    @notify_on_updated_label.setter
    def notify_on_updated_label(self, value):
        self._settings["project_preferences"]["notify_on_updated_label"] = "0" if value == False else "1"
        self._settings["preferences"]["notify_on_updated_label"] = "0" if value == False else "1"

    @property
    def slack_notifications_pref(self):
        return self._settings["preferences"]["slack_notifications_pref"]

    @slack_notifications_pref.setter
    def slack_notifications_pref(self, value):
        self._settings["preferences"]["slack_notifications_pref"] = "0" if value == False else "1"




from __future__ import annotations

from typing import TYPE_CHECKING

from fiopy.config import Config
from fiopy.models.base_asset import BaseAssetModel
from fiopy.models.comment import CommentModel
from fiopy.annotations.autowire import autowire
from fiopy.http.http_request import HttpRequest
from fiopy.models.asset_impression import AssetImpression

if TYPE_CHECKING:
    from fiopy.models.user import UserModel


class AssetModel(BaseAssetModel):
    def __init__(self, **kwargs):

        super().__init__(**kwargs)

        self._archive_from = kwargs.get("archive_from")
        self._asset_type = kwargs.get("asset_type")
        self._cover = kwargs.get("cover")

        self._downloads = kwargs.get("downloads")

        self._frame_cover = kwargs.get("frame_cover")
        self._frame_custom = kwargs.get("frame_custom")
        self._frame_thumb = kwargs.get("frame_thumb")

        self._h264_1080_best = kwargs.get("h264_1080_best")
        self._h264_2160 = kwargs.get("h264_2160")
        self._h264_360 = kwargs.get("h264_360")
        self._h264_540 = kwargs.get("h264_540")
        self._h264_720 = kwargs.get("h264_720")
        self._hls_manifest = kwargs.get("hls_manifest")
        self._image_full = kwargs.get("image_full")
        self._image_high = kwargs.get("image_high")
        self._image_small = kwargs.get("image_small")
        self._includes = kwargs.get("includes")
        self._original = kwargs.get("original")
        self._original_upload = kwargs.get("original_upload")
        self._page_full = kwargs.get("page_full")
        self._page_high = kwargs.get("page_high")
        self._page_small = kwargs.get("page_small")
        self._required_transcodes = kwargs.get("required_transcodes")
        self._required_transcodes = kwargs.get("required_transcodes")
        self._sha256 = kwargs.get("sha256")
        self._source = kwargs.get("source")
        self._status = kwargs.get("status")
        self._thumb = kwargs.get("thumb")
        self._thumb_540 = kwargs.get("thumb_540")
        self._thumb_scrub = kwargs.get("thumb_scrub")
        self._transcode_statuses = kwargs.get("transcode_statuses")
        self._transcoded_at = kwargs.get("transcoded_at")
        self._transcodes = kwargs.get("transcodes")
        self._upload_completed_at = kwargs.get("upload_completed_at")
        self._upload_urls = kwargs.get("upload_urls")
        self._uploaded_at = kwargs.get("uploaded_at")
        self._versions = kwargs.get("versions")
        self._view_count = kwargs.get("view_count")
        self._webm_1080_best = kwargs.get("webm_1080_best")
        self._webm_360 = kwargs.get("webm_360")
        self._webm_540 = kwargs.get("webm_540")
        self._webm_720 = kwargs.get("webm_720")

        super().cleanup_values()

    @property
    def archive_from(self):
        return self._archive_from

    @property
    def asset_type(self):
        return self._asset_type

    @property
    def cover(self):
        return self._cover

    @property
    def downloads(self):
        return self._downloads

    @property
    def frame_cover(self):
        return self._frame_cover

    @property
    def frame_custom(self):
        return self._frame_custom

    @property
    def frame_thumb(self):
        return self._frame_thumb

    @property
    def h264_1080_best(self):
        return self._h264_1080_best

    @property
    def h264_2160(self):
        return self._h264_2160

    @property
    def h264_360(self):
        return self._h264_360

    @property
    def h264_540(self):
        return self._h264_540

    @property
    def h264_720(self):
        return self._h264_720

    @property
    def hls_manifest(self):
        return self._hls_manifest

    @property
    def image_full(self):
        return self._image_full

    @property
    def image_high(self):
        return self._image_high

    @property
    def image_small(self):
        return self._image_small

    @property
    def includes(self):
        return self._includes

    @property
    def original(self):
        return self._original

    @property
    def original_upload(self):
        return self._original_upload

    @property
    def page_full(self):
        return self._page_full

    @property
    def page_high(self):
        return self._page_high

    @property
    def page_small(self):
        return self._page_small

    @property
    def required_transcodes(self):
        return self._required_transcodes

    @property
    def sha256(self):
        return self._sha256

    @property
    def source(self):
        return self._source

    @property
    def status(self):
        return self._status

    @property
    def thumb(self):
        return self._thumb

    @property
    def thumb_540(self):
        return self._thumb_540

    @property
    def thumb_scrub(self):
        return self._thumb_scrub

    @property
    def transcode_statuses(self):
        return self._transcode_statuses

    @property
    def transcoded_at(self):
        return self._transcoded_at

    @property
    def transcodes(self):
        return self._transcodes

    @property
    def upload_completed_at(self):
        return self._upload_completed_at

    @property
    def upload_urls(self):
        return self._upload_urls

    @property
    def uploaded_at(self):
        return self._uploaded_at

    @property
    def versions(self):
        return self._versions

    @property
    def view_count(self):
        return self._view_count

    @property
    def webm_1080_best(self):
        return self._webm_1080_best

    @property
    def webm_360(self):
        return self._webm_360

    @property
    def webm_540(self):
        return self._webm_540

    @property
    def webm_720(self):
        return self._webm_720

    @autowire(argmap={"asset_id": "id"})
    def delete_asset(self, *, asset_id: str = None, user: UserModel = None) -> bool:
        response = (
            HttpRequest.request(Config.api["v2"]["hostname"])
            .delete(Config.api["v2"]["endpoints"]["batch_assests"])
            .set_header("Authorization", user.session_id)
            .body({"batch": [{"id": asset_id}]})
            .send()
        )

        return response.body().value(key="error").is_empty()

    @autowire(argmap={"asset_id": "id"})
    def get_impressions(self, *, asset_id: str = None, user: UserModel = None):
        response = (
            HttpRequest.request(Config.api["v2"]["hostname"])
            .get(Config.api["v2"]["endpoints"]["get_asset_impressions"])
            .add_path_param("asset_id", asset_id)
            .set_header("Authorization", user.session_id)
            .send()
        )

        return response.to_models(model_clazz=AssetImpression)

    @autowire(argmap={"asset_id": "id"})
    def get_comments(self, *, asset_id: str = None, user: UserModel = None) -> CommentModel:
        response = (
            HttpRequest.request(Config.api["v2"]["hostname"])
            .get(Config.api["v2"]["endpoints"]["asset_comments"])
            .add_path_param("asset_id", asset_id)
            .set_header("Authorization", user.session_id)
            .send()
        )

        return response.to_models(model_clazz=CommentModel)

    @autowire(argmap={"asset_id": "id"})
    def create_comment(
        self,
        *,
        annotation: str = None,
        text: str = None,
        private: bool = False,
        timestamp: str = None,
        page: int = None,
        asset_id: str = None,
        user: UserModel = None,
    ) -> CommentModel:
        response = (
            HttpRequest.request(Config.api["v2"]["hostname"])
            .post(Config.api["v2"]["endpoints"]["asset_comments"])
            .add_path_param("asset_id", asset_id)
            .set_header("Authorization", user.session_id)
            .body({"annotation": annotation, "text": text, "private": private, "timestamp": timestamp, "page": page})
            .send()
        )

        return response.to_model(model_clazz=CommentModel)




import inspect
import functools
import fiopy

__all__ = ["autowire"]


def autowire(*, argmap={}):
    def wrapper(func):
        @functools.wraps(func)
        def wrapped(self, **kwargs):
            signature = inspect.signature(func)
            kwargs["user"] = kwargs.get("user") or fiopy.Fio.default_user()
            for name, value in signature.parameters.items():
                if not kwargs.get(name) and name != "self" and name != "user":
                    wired_value = None
                    if "." in argmap.get(name, name):
                        for name_part in argmap.get(name, name).split("."):
                            wired_value = getattr(
                                wired_value or self,
                                argmap.get(name_part, name_part),
                                None,
                            )
                    else:
                        wired_value = getattr(self, argmap.get(name, name), None)
                    kwargs[name] = wired_value or value.default

            return func(self, **kwargs)

        return wrapped

    return wrapper
